@@ -56,6 +56,7 @@ guint gegl_node_signals[GEGL_NODE_LAST_SIGNAL] = {0};
 static void            gegl_node_class_init           (GeglNodeClass *klass);
 static void            gegl_node_init                 (GeglNode      *self);
 static void            finalize                       (GObject       *self_object);
+static void            dispose                        (GObject       *self_object);
 static void            set_property                   (GObject       *gobject,
                                                        guint          prop_id,
                                                        const GValue  *value,
@@ -87,7 +88,7 @@ static void property_changed (GObject    *gobject,
                               GParamSpec *arg1,
                               gpointer    user_data);
 
-G_DEFINE_TYPE_WITH_CODE (GeglNode, gegl_node, GEGL_TYPE_GRAPH,
+G_DEFINE_TYPE_WITH_CODE (GeglNode, gegl_node, GEGL_TYPE_OBJECT,
                          G_IMPLEMENT_INTERFACE (GEGL_TYPE_VISITABLE,
                                                 visitable_init))
 
@@ -98,6 +99,7 @@ gegl_node_class_init (GeglNodeClass * klass)
   GObjectClass *gobject_class = G_OBJECT_CLASS (klass);
 
   gobject_class->finalize     = finalize;
+  gobject_class->dispose      = dispose;
   gobject_class->set_property = set_property;
   gobject_class->get_property = get_property;
 
@@ -139,6 +141,7 @@ gegl_node_init (GeglNode *self)
   self->operation   = NULL;
   self->enabled     = TRUE;
   self->is_graph    = FALSE;
+  self->children    = NULL;
 }
 
 static void
@@ -152,6 +155,16 @@ visitable_init (gpointer ginterface,
   visitable_class->needs_visiting = visitable_needs_visiting;
 }
 
+static void
+dispose (GObject *gobject)
+{
+  GeglNode *self = GEGL_NODE (gobject);
+
+  gegl_node_remove_children (self);
+
+  G_OBJECT_CLASS (gegl_node_parent_class)->dispose (gobject);
+}
+
 static void
 finalize (GObject *gobject)
 {
@@ -759,7 +772,7 @@ gegl_node_get_depends_on (GeglNode *self)
 
       if (source_node->is_graph)
         {
-          GeglNode *proxy = gegl_graph_output (source_node, "output");
+          GeglNode *proxy = gegl_node_get_output_proxy (source_node, "output");
 
           if (! g_list_find (depends_on, proxy))
              depends_on = g_list_append (depends_on, proxy);
@@ -1017,7 +1030,7 @@ gegl_node_get (GeglNode    *self,
   va_list        var_args;
 
   g_return_if_fail (GEGL_IS_NODE (self));
-  g_return_if_fail (GEGL_IS_GRAPH (self) || GEGL_IS_OPERATION (self->operation));
+  g_return_if_fail (self->is_graph || GEGL_IS_OPERATION (self->operation));
 
   va_start (var_args, first_property_name);
   gegl_node_get_valist (self, first_property_name, var_args);
@@ -1154,7 +1167,7 @@ gegl_node_get_valist (GeglNode    *self,
           {
             pspec = g_object_class_find_property (
                G_OBJECT_GET_CLASS (G_OBJECT (
-                   gegl_graph_output (self, "output")->operation)), property_name);
+                   gegl_node_get_output_proxy (self, "output")->operation)), property_name);
             if (!pspec)
               {
                 pspec = g_object_class_find_property (
@@ -1224,7 +1237,7 @@ gegl_node_set_property (GeglNode     *self,
         {
           g_warning ("set_property for graph,. hmm");
           /* FIXME: should this really be "input")? is_graph doesn't seem to be used,.. */
-          g_object_set_property (G_OBJECT (gegl_graph_input (self, "input")->operation),
+          g_object_set_property (G_OBJECT (gegl_node_get_input_proxy (self, "input")->operation),
                 property_name, value);
         }
       else
@@ -1256,7 +1269,7 @@ gegl_node_get_property (GeglNode    *self,
         !strcmp (property_name, "output"))
       {
           g_warning ("Eeek");
-          g_object_get_property (G_OBJECT (gegl_graph_output (self, "output")->operation),
+          g_object_get_property (G_OBJECT (gegl_node_get_output_proxy (self, "output")->operation),
                 property_name, value);
       }
     else
@@ -1573,8 +1586,23 @@ gegl_node_detect (GeglNode *root,
   g_warning ("gegl_node_detect: %i, %i", x, y);*/
 
   /* make sure the have rects are computed */
-  gegl_node_get_bounding_box (root); 
 
-  
-  return NULL;
+  /* FIXME: do not call this all the time! */
+  if (root)
+    {
+      gegl_node_get_bounding_box (root); 
+
+      if (root->operation)
+        return gegl_operation_detect (root->operation, x, y);
+      else
+        {
+          if (root->is_graph)
+            {
+              GeglNode *foo = gegl_node_get_output_proxy (root, "output");
+              if (foo && foo != root)
+                return gegl_node_detect (foo, x, y);
+            }
+        }
+    }
+  return root;
 }
